import { Excalidraw } from '@excalidraw/excalidraw';

import { useEffect, useRef, useState } from 'react';
import { BinaryFileData, ExcalidrawImperativeAPI } from '@excalidraw/excalidraw/types';
import { Languages, LogOut, Save } from 'lucide-react';
import { MainMenu, Sidebar, Footer } from '@excalidraw/excalidraw';
import { throttle } from 'lodash-es';
import { useMarkStore, store as StoreManager } from '../../store';
import { useShallow } from 'zustand/shallow';
import { useListenLang } from './hooks/listen-lang';
import { toast } from 'react-toastify';
import { hashFile } from '../../modules/hash-file';
import { uploadChunkV2 } from '../../modules/upload';
import { downloadFile, toFile } from '../../modules/to-file';

type ImageResource = {};
export const ImagesResources = (props: ImageResource) => {
  const [images, setImages] = useState<string[]>([]);
  useEffect(() => {
    initImages();
  }, []);
  const refFiles = useRef<any>({});
  const { api } = useMarkStore(
    useShallow((state) => {
      return {
        api: state.api,
      };
    }),
  );
  const initImages = async () => {
    if (!api) return;
    const res = api.getFiles();
    console.log('res from ImageResource', res);
    setImages(Object.keys(res));
    refFiles.current = res;
  };
  return (
    <div className='w-full h-full'>
      {images.map((image) => {
        const isUrl = refFiles.current[image]?.dataURL?.startsWith?.('http');
        const dataURL = refFiles.current[image]?.dataURL;
        return (
          <div className='flex items-center gap-2 w-full overflow-hidden p-2 m-2 border border-gray-200 rounded-md shadow ' key={image}>
            <img className='w-10 h-10 m-4' src={dataURL} alt='image' />
            <div className='flex flex-col gap-1'>
              {isUrl && <div className='text-xs  line-clamp-4 break-all'> {dataURL}</div>}
              <div className='text-xs'>{refFiles.current[image]?.name}</div>
              <div className='text-xs'>{refFiles.current[image]?.type}</div>
              <div className='text-xs'>{refFiles.current[image]?.id}</div>
            </div>
          </div>
        );
      })}
    </div>
  );
};

export const ExcalidrawExpand = (props: { onClose: () => void }) => {
  const { onClose } = props;
  const [docked, setDocked] = useState(false);
  const store = useMarkStore(
    useShallow((state) => {
      return {
        loading: state.loading,
        updateMark: state.updateMark,
        api: state.api,
      };
    }),
  );
  const { lang, setLang, isZh } = useListenLang();
  return (
    <>
      {store.loading && (
        <div className='w-full h-full flex items-center justify-center absolute top-0 left-0 z-10'>
          <div className='w-full h-full bg-black opacity-10 absolute top-0 left-0 z-1'></div>
        </div>
      )}
      <MainMenu>
        <MainMenu.Item
          onSelect={() => {
            store.updateMark();
          }}>
          <div className='flex items-center gap-2'>
            <Save />
            <span>{isZh ? '保存' : 'Save'}</span>
          </div>
        </MainMenu.Item>
        <MainMenu.DefaultItems.LoadScene />
        <MainMenu.DefaultItems.Export />
        <MainMenu.DefaultItems.SaveToActiveFile />
        <MainMenu.DefaultItems.SaveAsImage />
        {/* <MainMenu.DefaultItems.LiveCollaborationTrigger onSelect={() => {}} isCollaborating={false} /> */}
        {/* <MainMenu.DefaultItems.CommandPalette /> */}
        <MainMenu.DefaultItems.SearchMenu />
        <MainMenu.DefaultItems.Help />
        <MainMenu.DefaultItems.ClearCanvas />
        <MainMenu.Separator />
        <MainMenu.DefaultItems.ChangeCanvasBackground />
        <MainMenu.Separator />
        <MainMenu.DefaultItems.ToggleTheme />
        <MainMenu.Separator />

        <MainMenu.Item
          onSelect={(e) => {
            const newLang = lang === 'zh-CN' ? 'en' : 'zh-CN';
            setLang(newLang);
          }}>
          <div className='flex items-center gap-2'>
            <Languages />
            <span>{isZh ? 'English' : '中文'}</span>
          </div>
        </MainMenu.Item>
        {onClose && (
          <>
            <MainMenu.Separator />
            <MainMenu.Item onSelect={onClose}>
              <div className='flex items-center gap-2'>
                <LogOut />
                <span>{isZh ? '退出当前画布' : 'Exit current canvas'}</span>
              </div>
            </MainMenu.Item>
          </>
        )}
      </MainMenu>
      <Sidebar name='custom' docked={docked} onDock={setDocked}>
        <Sidebar.Header />
        <Sidebar.Tabs>
          <Sidebar.Tab tab='one'>{<ImagesResources />}</Sidebar.Tab>
          <Sidebar.Tab tab='two'>Tab two!</Sidebar.Tab>
          <Sidebar.TabTriggers>
            <Sidebar.TabTrigger tab='one'>Image Resources</Sidebar.TabTrigger>
            <Sidebar.TabTrigger tab='two'>Two</Sidebar.TabTrigger>
          </Sidebar.TabTriggers>
        </Sidebar.Tabs>
      </Sidebar>

      <Footer>
        <Sidebar.Trigger
          name='custom'
          tab='one'
          style={{
            marginLeft: '0.5rem',
            background: '#70b1ec',
            color: 'white',
          }}>
          {isZh ? '图片资源' : 'Image Resources'}
        </Sidebar.Trigger>
      </Footer>
    </>
  );
};
type CoreProps = {
  onClose: () => void;
  id: string;
};
export const Core = ({ onClose, id }: CoreProps) => {
  const ref = useRef<ExcalidrawImperativeAPI>(null);
  const { lang } = useListenLang();
  const uploadLoadingRef = useRef<boolean>(false);
  const store = useMarkStore(
    useShallow((state) => {
      return {
        id: state.id,
        loading: state.loading,
        getMark: state.getMark,
        api: state.api,
        setApi: state.setApi,
        getCache: state.getCache,
        updateMark: state.updateMark,
      };
    }),
  );
  const cacheDataRef = useRef<{
    elements: { [key: string]: number };
    filesObject: Record<string, any>;
  }>({
    elements: {},
    filesObject: {},
  });
  useEffect(() => {
    id && store.getMark(id);
  }, [id]);
  const onSave = throttle(async (elements, appState, filesObject) => {
    const { elements: cacheElements, filesObject: cacheFiles } = cacheDataRef.current;
    const _store = StoreManager.getStore(id);
    if (!_store) {
      console.error('Store not found for id:', id);
      return;
    }
    const { setCache, loading } = _store.getState();
    if (loading) {
      return;
    }
    if (uploadLoadingRef.current) return;
    let isChange = false;
    const elementsObj = elements.reduce((acc, e) => {
      acc[e.id] = e.version;
      return acc;
    }, {});
    if (JSON.stringify(elementsObj) !== JSON.stringify(cacheElements)) {
      isChange = true;
    }
    if (JSON.stringify(cacheFiles) !== JSON.stringify(filesObject)) {
      isChange = true;
      const files = Object.values(filesObject) as any as BinaryFileData[];
      uploadLoadingRef.current = true;
      for (const file of files) {
        if (file.dataURL.startsWith('data')) {
          const _file = toFile(file.dataURL, file.id);
          const res = (await uploadChunkV2(_file, {
            filename: file.id,
            directory: id,
          })) as any;
          if (res.code === 200) {
            toast.success('上传图片成功');
          } else {
            toast.error('上传图片失败');
          }

          const [upload] = res.data?.upload || [];
          if (upload) {
            filesObject[file.id] = {
              ...filesObject[file.id],
              dataURL: upload.path,
            };
          } else {
            toast.error('上传图片失败');
          }
        } else {
          continue;
        }
      }
      uploadLoadingRef.current = false;
    }
    console.log('onSave', elements, appState, filesObject, 'isChange', isChange);
    if (!isChange) {
      return;
    }
    cacheDataRef.current = { elements: elementsObj, filesObject };
    setCache({
      data: {
        elements,
        filesObject,
      },
    });
  }, 2000);
  return (
    <Excalidraw
      initialData={{}}
      onChange={(elements, appState, filesObject) => {
        if (store.loading) return;
        onSave(elements, appState, filesObject);
      }}
      langCode={lang || 'zh-CN'}
      renderTopRightUI={() => {
        return <div></div>;
      }}
      excalidrawAPI={async (api) => {
        ref.current = api;
        store.setApi(api);
        const cache = await store.getCache(id, true);
        if (!cache) return;
        const elementsObj = cache.elements.reduce((acc, e) => {
          acc[e.id] = e.version;
          return acc;
        }, {});
        cacheDataRef.current = {
          elements: elementsObj,
          filesObject: cache.filesObject,
        };
      }}
      generateIdForFile={async (file) => {
        // return dayjs().format('YYYY-MM-DD-HH-mm-ss') + '.' + file.type.split('/')[1];
        const hash = await hashFile(file);
        console.log('hash', hash, 'filetype', file.type);
        const fileId = hash + '.' + file.type.split('/')[1];
        console.log('fileId', fileId);

        return fileId;
      }}>
      <ExcalidrawExpand onClose={onClose} />
    </Excalidraw>
  );
};
